""" Access control testing
"""

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request
from tests.components.template_version_manager.fixtures.fixtures import (
    TemplateVersionManagerAccessControlFixtures,
)

fixture_template_vm = TemplateVersionManagerAccessControlFixtures()


# FIXME: missing tests where CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT is True


class TestVersionManagerGet(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_user_version_manager_as_anonymous_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get(self.fixture.user1_tvm.id, request=mock_request)

    def test_get_global_version_manager_as_anonymous_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get(self.fixture.global_tvm.id, request=mock_request)

    def test_get_own_version_manager_as_user_returns_version_manager(self):
        mock_request = create_mock_request(user=self.user1)
        version_manager = version_manager_api.get(
            self.fixture.user1_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user1_tvm)

    def test_global_version_manager_as_user_returns_version_manager(self):
        mock_request = create_mock_request(user=self.user1)
        version_manager = version_manager_api.get(
            self.fixture.global_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_other_users_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get(self.fixture.user2_tvm.id, request=mock_request)

    def test_get_any_version_manager_as_superuser_returns_version_manager(self):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.get(
            self.fixture.user1_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user1_tvm)
        version_manager = version_manager_api.get(
            self.fixture.user2_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user2_tvm)
        version_manager = version_manager_api.get(
            self.fixture.global_tvm.id, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_other_users_version_manager_as_staff_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get(self.fixture.user2_tvm.id, request=mock_request)


class TestVersionManagerGetByIdList(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_by_id_list_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_by_id_list(
                [str(self.fixture.user1_tvm.id)], request=mock_request
            )

    def test_get_by_id_list_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_by_id_list(
                [str(self.fixture.global_tvm.id)], request=mock_request
            )

    def test_get_by_id_list_user_version_manager_as_user_returns_version_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        version_managers = version_manager_api.get_by_id_list(
            [str(self.fixture.user1_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))

    def test_get_by_id_list_global_version_manager_as_user_returns_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        version_managers = version_manager_api.get_by_id_list(
            [str(self.fixture.global_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm in list(version_managers))

    def test_get_by_id_list_other_user_version_manager_as_user_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_by_id_list(
                [str(self.fixture.user2_tvm.id)], request=mock_request
            )

    def test_get_by_id_list_user_version_manager_as_staff_returns_version_manager(self):
        mock_request = create_mock_request(user=self.staff_user1)
        version_managers = version_manager_api.get_by_id_list(
            [str(self.fixture.user1_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))

    def test_get_by_id_list_global_version_manager_as_staff_returns_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        version_managers = version_manager_api.get_by_id_list(
            [str(self.fixture.global_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm in list(version_managers))

    def test_get_by_id_list_other_user_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_by_id_list(
                [str(self.fixture.user2_tvm.id)], request=mock_request
            )

    def test_get_by_id_list_user_version_manager_as_superuser_returns_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.superuser1)
        version_managers = version_manager_api.get_by_id_list(
            [str(self.fixture.user1_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))

    def test_get_by_id_list_global_version_manager_as_superuser_returns_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.superuser1)
        version_managers = version_manager_api.get_by_id_list(
            [str(self.fixture.global_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm not in list(version_managers))
        self.assertTrue(self.fixture.global_tvm in list(version_managers))

    def test_get_by_id_list_other_user_version_manager_as_superuser_returns_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.superuser1)
        version_managers = version_manager_api.get_by_id_list(
            [str(self.fixture.user2_tvm.id)], request=mock_request
        )
        self.assertTrue(self.fixture.user1_tvm not in list(version_managers))
        self.assertTrue(self.fixture.user2_tvm in list(version_managers))
        self.assertTrue(self.fixture.global_tvm not in list(version_managers))


class TestVersionManagerGetFromVersion(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_user_version_manager_from_version_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_from_version(
                self.fixture.user1_template, request=mock_request
            )

    def test_get_global_version_manager_from_version_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_from_version(
                self.fixture.global_template, request=mock_request
            )

    def test_get_own_version_manager_from_version_as_user_returns_version_manager(self):
        mock_request = create_mock_request(user=self.user1)
        version_manager = version_manager_api.get_from_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user1_tvm)

    def test_global_version_manager_from_version_as_user_returns_version_manager(self):
        mock_request = create_mock_request(user=self.user1)
        version_manager = version_manager_api.get_from_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_other_users_version_manager_from_version_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_from_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_get_any_version_manager_from_version_as_superuser_returns_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.get_from_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user1_tvm)
        version_manager = version_manager_api.get_from_version(
            self.fixture.user2_template, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.user2_tvm)
        version_manager = version_manager_api.get_from_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_other_users_version_manager_from_version_as_staff_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_from_version(
                self.fixture.user2_template, request=mock_request
            )


class TestVersionManagerDisable(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_disable_user_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(self.fixture.user1_tvm, request=mock_request)

    def test_anonymous_disable_global_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(self.fixture.global_tvm, request=mock_request)

    def test_user_can_disable_user_version_manager(self):
        self.fixture.user1_tvm.is_disabled = False
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.disable(self.fixture.user1_tvm, request=mock_request)
        self.assertTrue(self.fixture.user1_tvm.is_disabled)

    def test_user_disable_global_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(self.fixture.global_tvm, request=mock_request)

    def test_user_disable_other_user_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(self.fixture.user2_tvm, request=mock_request)

    def test_staff_can_disable_user_version_manager(self):
        self.fixture.user1_tvm.is_disabled = False
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.disable(self.fixture.user1_tvm, request=mock_request)
        self.assertTrue(self.fixture.user1_tvm.is_disabled)

    def test_staff_can_disable_global_version_manager(self):
        self.fixture.global_tvm.is_disabled = False
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.disable(self.fixture.global_tvm, request=mock_request)
        self.assertTrue(self.fixture.global_tvm.is_disabled)

    def test_staff_disable_other_user_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable(self.fixture.user2_tvm, request=mock_request)

    def test_superuser_can_disable_user_version_manager(self):
        self.fixture.user1_tvm.is_disabled = False
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable(self.fixture.user1_tvm, request=mock_request)
        self.assertTrue(self.fixture.user1_tvm.is_disabled)

    def test_superuser_can_disable_global_version_manager(self):
        self.fixture.global_tvm.is_disabled = False
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable(self.fixture.global_tvm, request=mock_request)
        self.assertTrue(self.fixture.global_tvm.is_disabled)

    def test_superuser_can_disable_other_user_version_manager(self):
        self.fixture.user2_tvm.is_disabled = False
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.disable(self.fixture.user2_tvm, request=mock_request)
        self.assertTrue(self.fixture.user2_tvm.is_disabled)


class TestVersionManagerRestore(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_restore_user_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(self.fixture.user1_tvm, request=mock_request)

    def test_anonymous_restore_global_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(self.fixture.global_tvm, request=mock_request)

    def test_user_can_restore_user_version_manager(self):
        self.fixture.user1_tvm.is_disabled = True
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.restore(self.fixture.user1_tvm, request=mock_request)
        self.assertFalse(self.fixture.user1_tvm.is_disabled)

    def test_user_restore_global_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(self.fixture.global_tvm, request=mock_request)

    def test_user_restore_other_user_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(self.fixture.user2_tvm, request=mock_request)

    def test_staff_can_restore_user_version_manager(self):
        self.fixture.user1_tvm.is_disabled = True
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.restore(self.fixture.user1_tvm, request=mock_request)
        self.assertFalse(self.fixture.user1_tvm.is_disabled)

    def test_staff_can_restore_global_version_manager(self):
        self.fixture.global_tvm.is_disabled = True
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.restore(self.fixture.global_tvm, request=mock_request)
        self.assertFalse(self.fixture.global_tvm.is_disabled)

    def test_staff_restore_other_user_version_manager_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore(self.fixture.user2_tvm, request=mock_request)

    def test_superuser_can_restore_user_version_manager(self):
        self.fixture.user1_tvm.is_disabled = True
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore(self.fixture.user1_tvm, request=mock_request)
        self.assertFalse(self.fixture.user1_tvm.is_disabled)

    def test_superuser_can_restore_global_version_manager(self):
        self.fixture.global_tvm.is_disabled = True
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore(self.fixture.global_tvm, request=mock_request)
        self.assertFalse(self.fixture.global_tvm.is_disabled)

    def test_superuser_can_restore_other_user_version_manager(self):
        self.fixture.user2_tvm.is_disabled = True
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.restore(self.fixture.user2_tvm, request=mock_request)
        self.assertFalse(self.fixture.user2_tvm.is_disabled)


class TestVersionManagerRestoreVersion(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_restore_version_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.user1_template, request=mock_request
            )

    def test_anonymous_restore_version_global_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_can_restore_version_user_version_manager(self):
        self.fixture.user1_tvm.disabled_versions = [str(self.fixture.user1_template.id)]
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager = version_manager_api.restore_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id) not in version_manager.disabled_versions
        )

    def test_user_restore_version_global_version_manager_raises_access_control_error(
        self,
    ):
        self.fixture.global_tvm.disabled_versions = [
            str(self.fixture.global_template.id)
        ]
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_restore_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_staff_can_restore_version_user_version_manager(self):
        self.fixture.user1_tvm.disabled_versions = [str(self.fixture.user1_template.id)]
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = version_manager_api.restore_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id) not in version_manager.disabled_versions
        )

    def test_staff_can_restore_version_global_version_manager(self):
        self.fixture.global_tvm.disabled_versions = [
            str(self.fixture.global_template.id)
        ]
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = version_manager_api.restore_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id)
            not in version_manager.disabled_versions
        )

    def test_staff_restore_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.restore_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_superuser_can_restore_version_user_version_manager(self):
        self.fixture.user1_tvm.disabled_versions = [str(self.fixture.user1_template.id)]
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.restore_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id) not in version_manager.disabled_versions
        )

    def test_superuser_can_restore_version_global_version_manager(self):
        self.fixture.global_tvm.disabled_versions = [
            str(self.fixture.global_template.id)
        ]
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.restore_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id)
            not in version_manager.disabled_versions
        )

    def test_superuser_can_restore_version_other_user_version_manager(self):
        self.fixture.user2_tvm.disabled_versions = [str(self.fixture.user2_template.id)]
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.restore_version(
            self.fixture.user2_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user2_template.id) not in version_manager.disabled_versions
        )


class TestVersionManagerDisableVersion(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_disable_version_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.user1_template, request=mock_request
            )

    def test_anonymous_disable_version_global_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_can_disable_version_user_version_manager(self):
        self.fixture.user1_tvm.current = "current_id"
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager = version_manager_api.disable_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id) in version_manager.disabled_versions
        )

    def test_user_disable_version_global_version_manager_raises_access_control_error(
        self,
    ):
        self.fixture.global_tvm.current = "current_id"
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.global_template, request=mock_request
            )

    def test_user_disable_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_staff_can_disable_version_user_version_manager(self):
        self.fixture.user1_tvm.current = "current_id"
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = version_manager_api.disable_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id) in version_manager.disabled_versions
        )

    def test_staff_can_disable_version_global_version_manager(self):
        self.fixture.global_tvm.current = "current_id"
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = version_manager_api.disable_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id) in version_manager.disabled_versions
        )

    def test_staff_disable_version_other_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.disable_version(
                self.fixture.user2_template, request=mock_request
            )

    def test_superuser_can_disable_version_user_version_manager(self):
        self.fixture.user1_tvm.current = "current_id"
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.disable_version(
            self.fixture.user1_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user1_template.id) in version_manager.disabled_versions
        )

    def test_superuser_can_disable_version_global_version_manager(self):
        self.fixture.global_tvm.current = "current_id"
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.disable_version(
            self.fixture.global_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.global_template.id) in version_manager.disabled_versions
        )

    def test_superuser_can_disable_version_other_user_version_manager(self):
        self.fixture.user2_tvm.current = "current_id"
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.disable_version(
            self.fixture.user2_template, request=mock_request
        )
        self.assertTrue(
            str(self.fixture.user2_template.id) in version_manager.disabled_versions
        )


class TestVersionManagerSetCurrent(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_anonymous_set_current_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.user1_template, request=mock_request
            )

    def test_anonymous_set_current_global_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.global_template, request=mock_request
            )

    def test_user_can_set_current_user_version_manager(self):
        self.fixture.user1_tvm.current = "current_id"
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        version_manager = version_manager_api.set_current(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(str(self.fixture.user1_template.id), version_manager.current)

    def test_user_set_current_global_version_manager_raises_access_control_error(self):
        self.fixture.global_tvm.current = "current_id"
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.global_template, request=mock_request
            )

    def test_user_set_current_other_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.user2_template, request=mock_request
            )

    def test_staff_can_set_current_user_version_manager(self):
        self.fixture.user1_tvm.current = "current_id"
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = version_manager_api.set_current(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(str(self.fixture.user1_template.id), version_manager.current)

    def test_staff_can_set_current_global_version_manager(self):
        self.fixture.global_tvm.current = "current_id"
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = version_manager_api.set_current(
            self.fixture.global_template, request=mock_request
        )
        self.assertEqual(str(self.fixture.global_template.id), version_manager.current)

    def test_staff_set_current_other_user_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.set_current(
                self.fixture.user2_template, request=mock_request
            )

    def test_superuser_can_set_current_user_version_manager(self):
        self.fixture.user1_tvm.current = "current_id"
        self.fixture.user1_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.set_current(
            self.fixture.user1_template, request=mock_request
        )
        self.assertEqual(str(self.fixture.user1_template.id), version_manager.current)

    def test_superuser_can_set_current_global_version_manager(self):
        self.fixture.global_tvm.current = "current_id"
        self.fixture.global_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.set_current(
            self.fixture.global_template, request=mock_request
        )
        self.assertEqual(str(self.fixture.global_template.id), version_manager.current)

    def test_superuser_can_set_current_other_user_version_manager(self):
        self.fixture.user2_tvm.current = "current_id"
        self.fixture.user2_tvm.save()
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = version_manager_api.set_current(
            self.fixture.user2_template, request=mock_request
        )
        self.assertEqual(str(self.fixture.user2_template.id), version_manager.current)


class TestVersionManagerUpsert(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_upsert_user_version_manager_as_anonymous_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(self.fixture.user1_tvm, request=mock_request)

    def test_upsert_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(self.fixture.global_tvm, request=mock_request)

    def test_upsert_own_version_manager_as_user_saves(self):
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.upsert(self.fixture.user1_tvm, request=mock_request)

    def test_upsert_other_users_version_manager_as_user_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(self.fixture.user2_tvm, request=mock_request)

    def test_upsert_global_version_manager_as_user_raises_access_control_error(self):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(self.fixture.global_tvm, request=mock_request)

    def test_upsert_own_version_manager_as_staff_saves(self):
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.upsert(self.fixture.user1_tvm, request=mock_request)

    def test_upsert_other_users_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.upsert(self.fixture.user2_tvm, request=mock_request)

    def test_upsert_global_version_manager_as_staff_saves(self):
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.upsert(self.fixture.global_tvm, request=mock_request)

    def test_upsert_own_version_manager_as_superuser_saves(self):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.upsert(self.fixture.user1_tvm, request=mock_request)

    def test_upsert_other_users_version_manager_as_superuser_saves(self):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.upsert(self.fixture.user2_tvm, request=mock_request)

    def test_upsert_global_version_manager_as_superuser_saves(self):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.upsert(self.fixture.global_tvm, request=mock_request)


class TestVersionManagerInsertVersion(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_insert_version_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.insert_version(
                self.fixture.user1_tvm,
                self.fixture.user1_template,
                request=mock_request,
            )

    def test_insert_version_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.insert_version(
                self.fixture.global_tvm,
                self.fixture.global_template,
                request=mock_request,
            )

    def test_insert_version_own_version_manager_as_user_saves(self):
        mock_request = create_mock_request(user=self.user1)
        version_manager_api.insert_version(
            self.fixture.user1_tvm, self.fixture.user1_template, request=mock_request
        )

    def test_insert_version_other_users_version_manager_as_user_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.insert_version(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )

    def test_insert_version_global_version_manager_as_user_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.insert_version(
                self.fixture.global_tvm,
                self.fixture.global_template,
                request=mock_request,
            )

    def test_insert_version_own_version_manager_as_staff_saves(self):
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.insert_version(
            self.fixture.user1_tvm, self.fixture.user1_template, request=mock_request
        )

    def test_insert_version_other_users_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.insert_version(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )

    def test_insert_version_global_version_manager_as_staff_saves(self):
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager_api.insert_version(
            self.fixture.global_tvm, self.fixture.global_template, request=mock_request
        )

    def test_insert_version_own_version_manager_as_superuser_saves(self):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.insert_version(
            self.fixture.user1_tvm, self.fixture.user1_template, request=mock_request
        )

    def test_insert_version_other_users_version_manager_as_superuser_saves(self):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.insert_version(
            self.fixture.user2_tvm, self.fixture.user2_template, request=mock_request
        )

    def test_insert_version_global_version_manager_as_superuser_saves(self):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager_api.insert_version(
            self.fixture.global_tvm, self.fixture.global_template, request=mock_request
        )


class TestVersionManagerGetActiveGlobalVersionManagerByTitle(
    MongoIntegrationBaseTestCase
):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_active_global_version_manager_by_title_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )

    def test_get_active_global_version_manager_as_user_returns_version_manager(self):
        mock_request = create_mock_request(user=self.user1)
        version_manager = (
            version_manager_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_active_global_version_manager_by_title_as_superuser_returns_version_manager(
        self,
    ):
        mock_request = create_mock_request(user=self.superuser1)
        version_manager = (
            version_manager_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)

    def test_get_active_global_version_manager_by_title_as_staff_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        version_manager = (
            version_manager_api.get_active_global_version_manager_by_title(
                self.fixture.global_tvm.title, request=mock_request
            )
        )
        self.assertEqual(version_manager, self.fixture.global_tvm)


class TestVersionManagerGetVersionNumber(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_version_number_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.user1_tvm,
                self.fixture.user1_template,
                request=mock_request,
            )

    def test_get_version_number_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.global_tvm,
                self.fixture.global_template,
                request=mock_request,
            )

    def test_get_version_number_own_version_manager_as_user_returns_version_number(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        version_number = version_manager_api.get_version_number(
            self.fixture.user1_tvm, self.fixture.user1_template.id, request=mock_request
        )
        self.assertEqual(version_number, 1)

    def test_global_version_manager_as_user_returns_version_number(self):
        mock_request = create_mock_request(user=self.user1)
        version_number = version_manager_api.get_version_number(
            self.fixture.global_tvm,
            self.fixture.global_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)

    def test_get_version_number_other_users_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )

    def test_get_version_number_any_version_manager_as_superuser_returns_version_number(
        self,
    ):
        mock_request = create_mock_request(user=self.superuser1)
        version_number = version_manager_api.get_version_number(
            self.fixture.user1_tvm, self.fixture.user1_template.id, request=mock_request
        )
        self.assertEqual(version_number, 1)
        version_number = version_manager_api.get_version_number(
            self.fixture.user2_tvm, self.fixture.user2_template.id, request=mock_request
        )
        self.assertEqual(version_number, 1)
        version_number = version_manager_api.get_version_number(
            self.fixture.global_tvm,
            self.fixture.global_template.id,
            request=mock_request,
        )
        self.assertEqual(version_number, 1)

    def test_get_version_number_other_users_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_number(
                self.fixture.user2_tvm,
                self.fixture.user2_template,
                request=mock_request,
            )


class TestVersionManagerGetVersionByNumber(MongoIntegrationBaseTestCase):

    fixture = fixture_template_vm

    def setUp(self):
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.staff_user1 = create_mock_user(user_id="1", is_staff=True)
        self.superuser1 = create_mock_user(user_id="1", is_superuser=True)
        self.fixture.insert_data()

    def test_get_version_by_number_user_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.user1_tvm, 1, request=mock_request
            )

    def test_get_version_by_number_global_version_manager_as_anonymous_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.anonymous_user)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.global_tvm, 1, request=mock_request
            )

    def test_get_version_by_number_own_version_manager_as_user_returns_version(self):
        mock_request = create_mock_request(user=self.user1)
        version = version_manager_api.get_version_by_number(
            self.fixture.user1_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.user1_template.id))

    def test_global_version_manager_as_user_returns_version(self):
        mock_request = create_mock_request(user=self.user1)
        version = version_manager_api.get_version_by_number(
            self.fixture.global_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.global_template.id))

    def test_get_version_by_number_other_users_version_manager_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.user2_tvm, 1, request=mock_request
            )

    def test_get_version_by_number_any_version_manager_as_superuser_returns_version(
        self,
    ):
        mock_request = create_mock_request(user=self.superuser1)
        version = version_manager_api.get_version_by_number(
            self.fixture.user1_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.user1_template.id))
        version = version_manager_api.get_version_by_number(
            self.fixture.user2_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.user2_template.id))
        version = version_manager_api.get_version_by_number(
            self.fixture.global_tvm, 1, request=mock_request
        )
        self.assertEqual(version, str(self.fixture.global_template.id))

    def test_get_version_by_number_other_users_version_manager_as_staff_raises_access_control_error(
        self,
    ):
        mock_request = create_mock_request(user=self.staff_user1)
        with self.assertRaises(AccessControlError):
            version_manager_api.get_version_by_number(
                self.fixture.user2_tvm, 1, request=mock_request
            )



""" Template tag to access a dictionary's key
"""
from django import template

register = template.Library()


# TODO: see if we can do it directly in the template
@register.filter(name="get")
def get(d, k):
    """Get the value of the dictionary d with the key k.
    Args:
        d: dictionary
        k: key

    Returns: the value

    """
    return d.get(k, None)



""" Serializers for Django Group
"""
from django.contrib.auth.models import Group
from rest_framework import serializers


class GroupSerializer(serializers.ModelSerializer):
    """Group serializer"""

    class Meta(object):
        """Meta"""

        model = Group
        fields = ("id", "name")



from unittest import TestCase

from django.urls import reverse
from django.test.utils import override_settings
from mock.mock import patch

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template import api as template_api
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request
from core_main_app.utils.xsd_flattener.xsd_flattener_database_url import (
    XSDFlattenerDatabaseOrURL,
    XSDFlattenerRequestsURL,
)


class TestXSDFlattenerDatabaseUrl(TestCase):
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch.object(XSDFlattenerRequestsURL, "get_dependency_content")
    def test_url_not_recognized_use_xsd_flattener_url(
        self, mock_get_dependency_content
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        xml_string = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:include schemaLocation="http://dummy.com/download?id=1234"/></xs:schema>'
        )
        dependency = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="test"/></xs:schema>'
        )
        mock_get_dependency_content.return_value = dependency

        # Act
        flattener = XSDFlattenerDatabaseOrURL(xml_string, request=mock_request)
        flat_string = flattener.get_flat()

        # Assert
        self.assertTrue('<xs:element name="test"/>' in flat_string)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch.object(template_api, "get")
    def test_url_recognized_use_database(self, mock_get):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        url_template_download = reverse(
            "core_main_app_rest_template_download", kwargs={"pk": "pk"}
        )
        xml_string = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:include schemaLocation="http://dummy.com{0}?id=1234"/>'
            "</xs:schema>".format(url_template_download)
        )
        dependency = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="test"/></xs:schema>'
        )
        mock_get.return_value = Template(content=dependency)

        # Act
        flattener = XSDFlattenerDatabaseOrURL(xml_string, request=mock_request)
        flat_string = flattener.get_flat()

        # Assert
        self.assertTrue('<xs:element name="test"/>' in flat_string)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch.object(template_api, "get")
    @patch.object(XSDFlattenerRequestsURL, "get_dependency_content")
    def test_url_recognized_template_does_not_exist(
        self, mock_get_dependency_content, mock_get
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        url_template_download = reverse(
            "core_main_app_rest_template_download", kwargs={"pk": "pk"}
        )
        xml_string = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:include schemaLocation="http://dummy.com{0}?id=1234"/>'
            "</xs:schema>".format(url_template_download)
        )
        dependency = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="test"/></xs:schema>'
        )
        mock_get_dependency_content.return_value = dependency
        mock_get.side_effect = DoesNotExist("Error")

        # Act
        flattener = XSDFlattenerDatabaseOrURL(xml_string, request=mock_request)
        flat_string = flattener.get_flat()

        # Assert
        self.assertTrue('<xs:element name="test"/>' in flat_string)



"""
Template API
"""
import logging

from core_main_app.access_control.api import is_superuser
from core_main_app.access_control.decorators import access_control
from core_main_app.commons import exceptions
from core_main_app.components.template.access_control import (
    can_read,
    can_write,
    get_accessible_owners,
    can_read_list,
)
from core_main_app.components.template.models import Template
from core_main_app.utils.xml import (
    is_schema_valid,
    get_hash,
    get_template_with_server_dependencies,
    get_local_dependencies,
)

logger = logging.getLogger(__name__)


@access_control(can_write)
def upsert(template, request):
    """Save or Updates the template.

    Args:
        template:
        request:

    Returns:

    """
    # Check if schema is valid
    is_schema_valid(template.content, request=request)
    # Get hash for the template
    template.hash = get_hash(template.content)
    # Register local dependencies
    _register_local_dependencies(template, request=request)
    # Save template
    return template.save()


@access_control(can_write)
def init_template_with_dependencies(template, dependencies_dict, request):
    """Initialize template content and dependencies from a dictionary.

    Args:
        template:
        dependencies_dict:
        request:

    Returns:

    """
    if dependencies_dict is not None:
        # update template content
        template.content = get_template_with_server_dependencies(
            template.content, dependencies_dict, request=request
        )

    return template


@access_control(can_write)
def set_display_name(template, display_name, request):
    """Set template display name.

    Args:
        template:
        display_name:
        request:

    Returns:

    """
    # Set display name
    template.display_name = display_name
    # Save
    template.save()


@access_control(can_read)
def get(template_id, request):
    """Get a template.

    Args:
        template_id:
        request:

    Returns:

    """
    return Template.get_by_id(template_id)


@access_control(can_read_list)
def get_all_accessible_by_hash(template_hash, request):
    """Return all template having the given hash.

    Args:
        template_hash: Template hash
        request:

    Returns:
        List of Template instance.

    """
    return Template.get_all_by_hash(
        template_hash, users=get_accessible_owners(request=request)
    )


@access_control(can_read_list)
def get_all_accessible_by_hash_list(template_hash_list, request):
    """Return all template having the given hash list.

    Args:
        template_hash_list: Template hash list.
        request:

    Returns:
        List of Template instance.

    """
    return Template.get_all_by_hash_list(
        template_hash_list, users=get_accessible_owners(request=request)
    )


@access_control(can_read_list)
def get_all_accessible_by_id_list(template_id_list, request):
    """Returns all template with id in list

    Args:
        template_id_list:
        request:

    Returns:

    """
    return Template.get_all_by_id_list(
        template_id_list, users=get_accessible_owners(request=request)
    )


@access_control(is_superuser)
def get_all(request, is_cls=True):
    """List all templates.

    Returns:

    """
    return Template.get_all(is_cls)


@access_control(can_write)
def delete(template, request):
    """Delete the template.

    Returns:

    """
    template.delete()


def _register_local_dependencies(template, request):
    """Register local dependencies for the given template.

    Args:
        template: Template instance.

    Returns:

    """
    # Clean all dependencies. Template content could have been changed.
    del template.dependencies
    # Get local dependencies
    local_dependencies = get_local_dependencies(template.content)
    for local_dependency in local_dependencies:
        try:
            # get the dependency
            dependency_object = get(local_dependency, request=request)
            # add the dependency
            template.dependencies.append(dependency_object)
        except exceptions.DoesNotExist as e:
            logger.warning(
                "Dependency {0} throw an exception: {1}".format(
                    local_dependency, str(e)
                )
            )



""" Login page api
"""
import core_main_app.components.web_page.api as web_page_api
from core_main_app.commons.enums import WEB_PAGE_TYPES
from core_main_app.commons.exceptions import ApiError

LOGIN_PAGE_NAME = "login"
LOGIN_PAGE_TYPE = WEB_PAGE_TYPES[LOGIN_PAGE_NAME]


def get():
    """Get the login page if exist

    Returns: login web page
    """
    return web_page_api.get(LOGIN_PAGE_NAME)


def upsert(login_page):
    """Post the login page

    Parameters:
        login_page (WebPage): Webpage for the login

    Returns: login web page
    """
    if login_page.type != LOGIN_PAGE_TYPE:
        raise ApiError(
            "Webpage type not coherent (expected: %s; actual %s"
            % (str(LOGIN_PAGE_TYPE), str(login_page.type))
        )

    return web_page_api.upsert(login_page)



"""
Version Manager unit tests
"""
from unittest.case import TestCase

from bson.objectid import ObjectId
from mock.mock import Mock, patch
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.components.version_manager.models import VersionManager
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request


class TestVersionManagerGet(TestCase):
    @patch("core_main_app.components.version_manager.models.VersionManager.get_by_id")
    def test_version_manager_get_returns_version_manager(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_version_manager = _create_mock_version_manager(title="Schema")

        mock_get_by_id.return_value = mock_version_manager

        # Act
        result = version_manager_api.get(mock_version_manager.id, request=mock_request)

        # Assert
        self.assertIsInstance(result, VersionManager)

    @patch("core_main_app.components.version_manager.models.VersionManager.get_by_id")
    def test_version_manager_get_raises_exception_if_object_does_not_exist(
        self, mock_get_by_id
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_absent_id = ObjectId()
        mock_get_by_id.side_effect = mongoengine_errors.DoesNotExist

        # Act + Assert
        with self.assertRaises(mongoengine_errors.DoesNotExist):
            version_manager_api.get(mock_absent_id, request=mock_request)


class TestVersionManagerGetFromVersionId(TestCase):
    @patch("core_main_app.components.version_manager.models.VersionManager.get_all")
    def test_version_manager_get_from_version_id_returns_version_manager(
        self, mock_get_all
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_version_manager_1 = _create_mock_version_manager(title="Schema1")
        version = _create_mock_object()
        mock_version_manager_1.versions.append(str(version.id))

        mock_version_manager_2 = _create_mock_version_manager(title="Schema2")

        mock_get_all.return_value = [mock_version_manager_1, mock_version_manager_2]

        # Act
        result = version_manager_api.get_from_version(version, request=mock_request)

        # Assert
        self.assertIsInstance(result, VersionManager)

    @patch("core_main_app.components.version_manager.models.VersionManager.get_all")
    def test_version_manager_get_from_version_id_throws_exception_if_id_absent(
        self, mock_get_all
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_version_manager_1 = _create_mock_version_manager(title="Schema1")
        mock_version_manager_2 = _create_mock_version_manager(title="Schema2")

        mock_get_all.return_value = [mock_version_manager_1, mock_version_manager_2]

        mock_absent = _create_mock_object()

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.get_from_version(mock_absent, request=mock_request)


class TestVersionManagerRestoreVersion(TestCase):
    @patch("core_main_app.components.version_manager.models.VersionManager.get_all")
    def test_version_manager_get_raises_exception_if_object_does_not_exist(
        self, mock_get_all
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_absent = _create_mock_object()
        mock_version_manager = _create_mock_version_manager()

        mock_get_all.return_value = [mock_version_manager]

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.restore_version(mock_absent, request=mock_request)


class TestVersionManagerDisableVersion(TestCase):
    @patch("core_main_app.components.version_manager.models.VersionManager.get_all")
    def test_version_manager_disable_version_raises_exception_if_object_does_not_exist(
        self, mock_get_all
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_absent = _create_mock_object()
        mock_version_manager = _create_mock_version_manager()

        mock_get_all.return_value = [mock_version_manager]

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.disable_version(mock_absent, request=mock_request)

    @patch("core_main_app.components.version_manager.models.VersionManager.get_all")
    def test_version_manager_disable_current_version_throws_exception(
        self, mock_get_all
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_version_manager = _create_mock_version_manager()
        mock_current = _create_mock_object()
        mock_version_manager.versions.append(str(mock_current.id))
        mock_version_manager.current = str(mock_current.id)

        mock_get_all.return_value = [mock_version_manager]

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.disable_version(mock_current, request=mock_request)

    @patch("core_main_app.components.version_manager.models.VersionManager.get_all")
    def test_version_manager_disable_version_raises_exception_if_new_current_does_not_exist(
        self, mock_get_all
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_version_manager = _create_mock_version_manager()
        mock_absent = _create_mock_object()
        mock_current = _create_mock_object()
        mock_version_manager.versions.append(str(mock_current.id))
        mock_version_manager.current = str(mock_current.id)

        mock_get_all.return_value = [mock_version_manager]

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.disable_version(
                mock_current, request=mock_request, new_current=mock_absent
            )


class TestVersionManagerSetCurrent(TestCase):
    @patch("core_main_app.components.version_manager.models.VersionManager.get_all")
    @patch(
        "core_main_app.components.version_manager.models.VersionManager.get_disabled_versions"
    )
    def test_version_manager_set_current_raises_api_error_if_disabled(
        self, mock_get_disabled_versions, mock_get_all
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        to_disable = _create_mock_object()
        mock_vm = _create_mock_version_manager()

        mock_get_all.return_value = [mock_vm]
        mock_get_disabled_versions.return_value = [str(to_disable.id)]

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            version_manager_api.set_current(to_disable, request=mock_request)


class TestVersionManagerInsert(TestCase):
    @patch("core_main_app.components.version_manager.models.VersionManager.save")
    def test_version_manager_insert_first_version_adds_versions(self, mock_save):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        version_manager = _create_version_manager()
        version = _create_mock_object()
        mock_save.return_value = None

        # Act + Assert
        version_manager_api.insert_version(
            version_manager, version, request=mock_request
        )

        self.assertEquals(version_manager.versions[0], str(version.id))

    @patch("core_main_app.components.version_manager.models.VersionManager.save")
    def test_version_manager_insert_first_version_sets_current(self, mock_save):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        version_manager = _create_version_manager()
        version = _create_mock_object()
        mock_save.return_value = None

        # Act + Assert
        version_manager_api.insert_version(
            version_manager, version, request=mock_request
        )

        self.assertEquals(version_manager.current, str(version.id))


class TestVersionManagerGetActiveGlobalVersionManagerByTitle(TestCase):
    @patch.object(VersionManager, "get_active_global_version_manager_by_title")
    def test_version_manager_get_returns_version_manager(self, mock_get_active_global):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        title = "Schema"
        mock_version_manager = _create_mock_version_manager(title=title)

        mock_get_active_global.return_value = mock_version_manager

        # Act
        result = version_manager_api.get_active_global_version_manager_by_title(
            title, request=mock_request
        )

        # Assert
        self.assertIsInstance(result, VersionManager)

    @patch.object(VersionManager, "get_active_global_version_manager_by_title")
    def test_version_manager_get_raises_exception_if_object_does_not_exist(
        self, mock_get_active_global
    ):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(mock_user)
        mock_absent_title = "Schema"
        mock_get_active_global.side_effect = mongoengine_errors.DoesNotExist

        # Act + Assert
        with self.assertRaises(mongoengine_errors.DoesNotExist):
            version_manager_api.get_active_global_version_manager_by_title(
                mock_absent_title, request=mock_request
            )


class TestVersionManagerGetVersionByNumber(TestCase):
    def test_version_manager_get_returns_version(self):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        version = str(ObjectId())
        version_manager = _create_version_manager("Schema1", [version])

        # Act
        result = version_manager_api.get_version_by_number(
            version_manager, 1, request=mock_request
        )

        # Assert
        self.assertEquals(result, version)

    def test_version_manager_get_raises_exception_if_object_does_not_exist(self):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        version = str(ObjectId())
        version_manager = _create_version_manager("Schema1", [version])

        # Act + Assert
        with self.assertRaises(IndexError):
            version_manager_api.get_version_by_number(
                version_manager, 2, request=mock_request
            )


class TestVersionManagerGetVersionNumber(TestCase):
    def test_version_manager_get_returns_number(self):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        version = str(ObjectId())
        version_manager = _create_version_manager("Schema1", [version])

        # Act
        result = version_manager_api.get_version_number(
            version_manager, version, request=mock_request
        )

        # Assert
        self.assertEquals(result, 1)

    def test_version_manager_get_raises_exception_if_object_does_not_exist(self):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_request = create_mock_request(user=mock_user)
        version = str(ObjectId())
        absent_version = str(ObjectId())
        version_manager = _create_version_manager("Schema1", [version])

        # Act + Assert
        with self.assertRaises(ValueError):
            version_manager_api.get_version_number(
                version_manager, absent_version, request=mock_request
            )


def _create_mock_version_manager(title=""):
    """
    Create a mock version manager
    :param title:
    :return:
    """
    mock_vm = Mock(spec=VersionManager)
    mock_vm.title = title
    mock_vm.current = ObjectId()
    mock_vm.versions = [str(mock_vm.current)]
    mock_vm.id = ObjectId()
    return mock_vm


def _create_mock_object():
    """
    Create a mock object
    :return:
    """
    mock_object = Mock()
    mock_object.id = ObjectId()
    return mock_object


def _create_version_manager(title="", versions=[]):
    """
    Returns a templates version manager
    :param title:
    :return:
    """
    return VersionManager(
        id=ObjectId(), title=title, versions=versions, disabled_versions=[]
    )



""" Integration Test for Blob Rest API
"""
import unittest
from os.path import join, dirname, abspath

from django.test import override_settings
from mock import patch
from rest_framework import status

from core_main_app.components.blob import api as blob_api
from core_main_app.components.blob.models import Blob
from core_main_app.components.workspace.models import Workspace
from core_main_app.rest.blob import views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.blob.fixtures.fixtures import (
    BlobFixtures,
    AccessControlBlobFixture,
)

RESOURCES_PATH = join(dirname(abspath(__file__)), "data")
fixture_blob = BlobFixtures()
fixture_blob_workspace = AccessControlBlobFixture()


class TestBlobListAdmin(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(views.BlobListAdmin.as_view(), user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_as_superuser_returns_all_blobs(self):
        # Arrange
        user = create_mock_user("1", is_staff=True, is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobListAdmin.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 3)


class TestBlobList(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobList, self).setUp()
        self.data = {
            "filename": "file.txt",
            "blob": open(join(RESOURCES_PATH, "test.txt"), "r"),
        }

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(), user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_all_user_blobs(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_as_superuser_returns_all_user_blobs(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_correct_name_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(),
            user,
            data={"filename": self.fixture.blob_1.filename},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_correct_name_returns_correct_blob(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(),
            user,
            data={"filename": self.fixture.blob_1.filename},
        )

        # Assert
        self.assertEqual(response.data[0]["filename"], self.fixture.blob_1.filename)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_incorrect_name_returns_http_200(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(), user, data={"filename": "incorrect"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_incorrect_name_returns_empty_list(self):
        # Arrange
        user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_get(
            views.BlobList.as_view(), user, data={"filename": "incorrect"}
        )

        # Assert
        self.assertEqual(len(response.data), 0)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            views.BlobList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_adds_an_entry_in_database(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_post(
            views.BlobList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(len(blob_api.get_all()), 4)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_incorrect_file_parameter_returns_http_400(self):
        # Arrange
        user = create_mock_user("1")
        self.data["blob"] = "test.txt"

        # Act
        response = RequestMock.do_request_post(
            views.BlobList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestBlobDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobDetail, self).setUp()

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(), user, param={"pk": str(self.fixture.blob_1.id)}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_blob(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(), user, param={"pk": str(self.fixture.blob_1.id)}
        )

        # Assert
        self.assertEqual(response.data["filename"], self.fixture.blob_1.filename)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(), user, param={"pk": "507f1f77bcf86cd799439011"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_other_user_blob_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDetail.as_view(), user, param={"pk": str(self.fixture.blob_1.id)}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_returns_http_204(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(), user, param={"pk": str(self.fixture.blob_1.id)}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_deletes_one_blob_from_database(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(), user, param={"pk": str(self.fixture.blob_1.id)}
        )

        # Assert
        self.assertEqual(len(blob_api.get_all()), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(), user, param={"pk": "507f1f77bcf86cd799439011"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_other_user_blob_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(), user, param={"pk": str(self.fixture.blob_1.id)}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_other_user_as_superuser_returns_http_204(self):
        # Arrange
        user = create_mock_user("2", is_superuser=True)

        # Act
        response = RequestMock.do_request_delete(
            views.BlobDetail.as_view(), user, param={"pk": str(self.fixture.blob_1.id)}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobDownload(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobDownload, self).setUp()
        self.blob = open(join(RESOURCES_PATH, "test.txt"), "r").read()

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDownload.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDownload.as_view(), user, param={"pk": "507f1f77bcf86cd799439011"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_other_user_blob_returns_http_200(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_get(
            views.BlobDownload.as_view(),
            user,
            param={"pk": str(self.fixture.blob_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobDeleteList(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobDeleteList, self).setUp()
        self.data = [
            {"id": str(self.fixture.blob_1.id)},
            {"id": str(self.fixture.blob_2.id)},
        ]

    def test_post_a_list_containing_other_user_blob_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.BlobDeleteList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @unittest.skip("need to mock GridFS Blob Host")
    def test_post_a_list_containing_other_user_blob_as_superuser_returns_http_204(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            views.BlobDeleteList.as_view(), user, data=self.data
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobAssign(MongoIntegrationBaseTestCase):
    fixture = fixture_blob_workspace

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200(self, blob_get_by_id, workspace_get_by_id):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_1

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_updates_workspace(
        self, blob_get_by_id, workspace_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(str(blob.workspace.id), str(self.fixture.workspace_2.id))

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_returns_http_200(
        self, blob_get_by_id, workspace_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    def test_assign_bad_blob_id_returns_http_404(self, workspace_get_by_id):
        # Arrange
        fake_blob_id = "507f1f77bcf86cd799439011"
        user = create_mock_user("1", is_superuser=True)
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": fake_blob_id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(Blob, "get_by_id")
    def test_assign_bad_workspace_id_returns_http_404(self, blob_get_by_id):
        # Arrange
        fake_workspace_id = "507f1f77bcf86cd799439011"
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": fake_workspace_id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestBlobAssign(MongoIntegrationBaseTestCase):
    fixture = fixture_blob_workspace

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200(self, blob_get_by_id, workspace_get_by_id):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_1

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_updates_workspace(
        self, blob_get_by_id, workspace_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(str(blob.workspace.id), str(self.fixture.workspace_2.id))

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_returns_http_200(
        self, blob_get_by_id, workspace_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    def test_assign_bad_blob_id_returns_http_404(self, workspace_get_by_id):
        # Arrange
        fake_blob_id = "507f1f77bcf86cd799439011"
        user = create_mock_user("1", is_superuser=True)
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": fake_blob_id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(Blob, "get_by_id")
    def test_assign_bad_workspace_id_returns_http_404(self, blob_get_by_id):
        # Arrange
        fake_workspace_id = "507f1f77bcf86cd799439011"
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": fake_workspace_id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestBlobAssign(MongoIntegrationBaseTestCase):
    fixture = fixture_blob_workspace

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200(self, blob_get_by_id, workspace_get_by_id):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_1

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_updates_workspace(
        self, blob_get_by_id, workspace_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(str(blob.workspace.id), str(self.fixture.workspace_2.id))

    @patch.object(Workspace, "get_by_id")
    @patch.object(Blob, "get_by_id")
    def test_assign_blob_to_workspace_returns_http_200(
        self, blob_get_by_id, workspace_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Workspace, "get_by_id")
    def test_assign_bad_blob_id_returns_http_404(self, workspace_get_by_id):
        # Arrange
        fake_blob_id = "507f1f77bcf86cd799439011"
        user = create_mock_user("1", is_superuser=True)
        workspace_get_by_id.return_value = self.fixture.workspace_2

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": fake_blob_id, "workspace_id": self.fixture.workspace_2.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(Blob, "get_by_id")
    def test_assign_bad_workspace_id_returns_http_404(self, blob_get_by_id):
        # Arrange
        fake_workspace_id = "507f1f77bcf86cd799439011"
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user = create_mock_user(blob.user_id, is_superuser=True)
        blob_get_by_id.return_value = blob

        # Act
        response = RequestMock.do_request_patch(
            views.BlobAssign.as_view(),
            user,
            param={"pk": blob.id, "workspace_id": fake_workspace_id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestBlobChangeOwner(MongoIntegrationBaseTestCase):
    fixture = fixture_blob_workspace

    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200_if_user_is_superuser(
        self, blob_get_by_id, user_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user_request = create_mock_user("65467", is_staff=True, is_superuser=True)
        user_new_owner = create_mock_user("123")
        blob_get_by_id.return_value = blob
        user_get_by_id.return_value = user_new_owner

        # Act
        response = RequestMock.do_request_patch(
            views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": blob.id, "user_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_200_if_user_is_not_superuser_but_have_access_to_the_blob(
        self, blob_get_by_id, user_get_by_id
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user_request = create_mock_user("1", is_staff=True)
        user_new_owner = create_mock_user("123")
        blob_get_by_id.return_value = blob
        user_get_by_id.return_value = user_new_owner

        # Act
        response = RequestMock.do_request_patch(
            views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": blob.id, "user_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_get_returns_http_403_if_user_is_not_superuser_or_have_no_access_to_the_blob(
        self, blob_get_by_id, user_get_by_id, read_access_mock
    ):
        # Arrange
        blob = self.fixture.blob_collection[self.fixture.USER_1_WORKSPACE_1]
        user_request = create_mock_user("65467", is_staff=True)
        user_new_owner = create_mock_user("123")
        blob_get_by_id.return_value = blob
        user_get_by_id.return_value = user_new_owner
        read_access_mock.return_value = []

        # Act
        response = RequestMock.do_request_patch(
            views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": blob.id, "user_id": self.fixture.workspace_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



""" Access control testing for Blob.
"""
import unittest

from mock.mock import patch

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.blob import api as blob_api
from core_main_app.components.blob.models import Blob
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests.components.blob.fixtures.fixtures import AccessControlBlobFixture

fixture_blob = AccessControlBlobFixture()


class TestBlobGetById(MongoIntegrationBaseTestCase):

    fixture = fixture_blob

    def test_get_by_id_owner_with_read_access_returns_blob(self):
        blob_id = self.fixture.blob_collection[fixture_blob.USER_1_WORKSPACE_1].id
        mock_user = _create_user("1")
        blob = blob_api.get_by_id(blob_id, mock_user)
        self.assertTrue(isinstance(blob, Blob))

    def test_get_by_id_owner_without_read_access_returns_blob(self):
        blob_id = self.fixture.blob_collection[fixture_blob.USER_1_WORKSPACE_1].id
        mock_user = _create_user("1")
        with self.assertRaises(AccessControlError):
            blob_api.get_by_id(blob_id, mock_user)

    def test_get_by_id_owner_without_read_access_returns_blob(self):
        blob_id = self.fixture.blob_collection[fixture_blob.USER_1_WORKSPACE_1].id
        mock_user = _create_user("1")
        blob = blob_api.get_by_id(blob_id, mock_user)
        self.assertTrue(isinstance(blob, Blob))

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_by_id_user_without_read_access_raises_error(
        self, get_all_workspaces_with_read_access_by_user
    ):
        blob_id = self.fixture.blob_collection[fixture_blob.USER_1_WORKSPACE_1].id
        get_all_workspaces_with_read_access_by_user.return_value = []
        mock_user = _create_user("2")
        with self.assertRaises(AccessControlError):
            blob_api.get_by_id(blob_id, mock_user)

    def test_get_by_id_owner_no_workspace_read_access_returns_blob(self):
        blob_id = self.fixture.blob_collection[fixture_blob.USER_1_NO_WORKSPACE].id
        mock_user = _create_user("1")
        blob = blob_api.get_by_id(blob_id, mock_user)
        self.assertTrue(isinstance(blob, Blob))

    def test_get_by_id_not_owner_no_workspace_raises_error(self):
        blob_id = self.fixture.blob_collection[fixture_blob.USER_1_NO_WORKSPACE].id
        mock_user = _create_user("2")
        with self.assertRaises(AccessControlError):
            blob_api.get_by_id(blob_id, mock_user)


class TestBlobGetAll(MongoIntegrationBaseTestCase):

    fixture = fixture_blob

    def test_get_all_as_superuser_returns_all_blob(self):
        mock_user = _create_user("1", is_superuser=True)
        data_list = blob_api.get_all(mock_user)
        self.assertTrue(len(data_list) == len(self.fixture.blob_collection))

    def test_get_all_as_user_raises_error(self):
        mock_user = _create_user("1")
        with self.assertRaises(AccessControlError):
            blob_api.get_all(mock_user)


class TestBlobGetAllByUser(MongoIntegrationBaseTestCase):

    fixture = fixture_blob

    def test_get_all_by_user_returns_owned_blob(self):
        mock_user = _create_user("1")
        blob_list = blob_api.get_all_by_user(mock_user)
        self.assertTrue(len(blob_list) == 2)
        self.assertTrue(blob.id == "1" for blob in blob_list)

    def test_get_all_by_user_returns_no_blob_if_owns_zero(self):
        mock_user = _create_user("3")
        blob_list = blob_api.get_all_by_user(mock_user)
        self.assertTrue(len(blob_list) == 0)

    def test_get_all_by_user_as_superuser_returns_own_blob(self):
        mock_user = _create_user("1", is_superuser=True)
        blob_list = blob_api.get_all_by_user(mock_user)
        self.assertTrue(len(blob_list) == 2)
        self.assertTrue(blob.user_id == "1" for blob in blob_list)


class TestBlobGetAllByWorkspace(MongoIntegrationBaseTestCase):

    fixture = fixture_blob

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_all_by_workspace_returns_owned_blob(
        self,
        get_all_workspaces_with_write_access_by_user,
        get_all_workspaces_with_read_access_by_user,
    ):
        mock_user = _create_user(self.fixture.USER_1_WORKSPACE_1)
        get_all_workspaces_with_read_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        blob_list = blob_api.get_all_by_workspace(self.fixture.workspace_1, mock_user)
        self.assertTrue(blob.user_id == mock_user.id for blob in blob_list)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_all_by_workspace_returns_no_blob_if_owns_zero(
        self,
        get_all_workspaces_with_write_access_by_user,
        get_all_workspaces_with_read_access_by_user,
    ):
        mock_user = _create_user("7")
        get_all_workspaces_with_write_access_by_user.return_value = []
        get_all_workspaces_with_read_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.get_all_by_workspace(self.fixture.workspace_1, mock_user)

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user"
    )
    def test_get_all_by_workspace_as_superuser_returns_owned_blob(
        self,
        get_all_workspaces_with_write_access_by_user,
        get_all_workspaces_with_read_access_by_user,
    ):
        mock_user = _create_user("1")
        get_all_workspaces_with_read_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        get_all_workspaces_with_write_access_by_user.return_value = [
            self.fixture.workspace_1
        ]
        blob_list = blob_api.get_all_by_workspace(self.fixture.workspace_1, mock_user)
        self.assertTrue(blob.user_id == mock_user.id for blob in blob_list)


class TestBlobDelete(MongoIntegrationBaseTestCase):

    fixture = fixture_blob

    @unittest.skip("GridFS not supported by mongomock")
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_own_blob_in_accessible_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_1
        ]
        blob_api.delete(
            fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1], mock_user
        )

    @unittest.skip("GridFS not supported by mongomock")
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_own_blob_in_not_accessible_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        blob_api.delete(
            fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1], mock_user
        )

    @unittest.skip("GridFS not supported by mongomock")
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_others_blob_in_accessible_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_2
        ]
        blob_api.delete(
            fixture_blob.blob_collection[fixture_blob.USER_2_WORKSPACE_2], mock_user
        )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_others_blob_not_accessible_workspace_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = [
            fixture_blob.workspace_1
        ]
        with self.assertRaises(AccessControlError):
            blob_api.delete(
                fixture_blob.blob_collection[fixture_blob.USER_2_WORKSPACE_2], mock_user
            )

    @unittest.skip("GridFS not supported by mongomock")
    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_own_blob_not_in_workspace_deletes_blob(
        self, get_all_workspaces_with_write_access_by_user
    ):
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.delete(
                fixture_blob.blob_collection[fixture_blob.USER_1_WORKSPACE_1], mock_user
            )

    @patch(
        "core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user"
    )
    def test_delete_others_blob_not_in_workspace_raises_error(
        self, get_all_workspaces_with_write_access_by_user
    ):
        mock_user = _create_user("1")
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            blob_api.delete(
                fixture_blob.blob_collection[fixture_blob.USER_2_NO_WORKSPACE],
                mock_user,
            )


class TestBlobChangeOwner(MongoIntegrationBaseTestCase):

    fixture = fixture_blob

    def test_change_owner_from_owner_to_owner_ok(self):
        mock_owner = _create_user("1")
        blob_api.change_owner(
            document=fixture_blob.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
            new_user=mock_owner,
            user=mock_owner,
        )

    def test_change_owner_from_owner_to_user_ok(self):
        mock_owner = _create_user("1")
        mock_user = _create_user("2")
        blob_api.change_owner(
            document=fixture_blob.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
            new_user=mock_user,
            user=mock_owner,
        )

    def test_change_owner_from_user_to_user_raises_exception(self):
        mock_owner = _create_user("1")
        mock_user = _create_user("2")
        with self.assertRaises(AccessControlError):
            blob_api.change_owner(
                document=fixture_blob.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
                new_user=mock_owner,
                user=mock_user,
            )

    def test_change_owner_as_superuser_ok(self):
        mock_user = _create_user("2", is_superuser=True)
        blob_api.change_owner(
            document=fixture_blob.blob_collection[fixture_blob.USER_1_NO_WORKSPACE],
            new_user=mock_user,
            user=mock_user,
        )


def _create_user(user_id, is_superuser=False):
    return create_mock_user(user_id, is_superuser=is_superuser)



""" Constants
"""

MARKDOWN_UNSAFE = 0
MARKDOWN_GENERATION_FAILED = 1

MARKDOWN_ERRORS = [
    "Unprotected script tags have been detected. Please remove them to be able to save your work!",
    "HTML is not generated properly, please check your Markdown and HTML syntax!",
]

UNKNOWN_ERROR = (
    "An unknown error occured, please contact your administrator for more information."
)



""" Blobs downloader utils
"""
from logging import getLogger
from urllib.parse import urlparse

from core_main_app.commons import exceptions
from core_main_app.settings import SERVER_URI
from core_main_app.utils.requests_utils import requests_utils
from django.conf import settings

logger = getLogger(__name__)


class BlobDownloader:
    """Blob Downloader Class"""

    def __init__(self, url, session_key=""):
        """Blob Downloader Constructor

        Args:
            url: blob's url
            session_key: session key if needed
        """
        self.url = url
        self.url_base = self.get_url_base()
        self.session_key = session_key

    def get_blob_response(self):
        """get the blob response from local or remote instance

        Returns: Http Response

        """
        if self.is_url_from_local_instance():
            # call the local instance giving sessionid through the call
            return requests_utils.send_get_request(
                url=self.url, cookies={"sessionid": self.session_key}
            )
        else:
            # so it can be from a federated instance
            if "core_federated_search_app" in settings.INSTALLED_APPS:
                # import the api where we need to
                import core_federated_search_app.components.instance.api as instance_api

                try:
                    return instance_api.get_blob_response_from_url(
                        self.url_base, self.url
                    )
                except exceptions.DoesNotExist as e:
                    logger.info("BlobDownloader: The blob's url is not a known source")
            else:
                logger.info("BlobDownloader: core_federated_search_app is required")

        # here is the case where the blob come from an unidentified source
        raise exceptions.BlobDownloaderError("Blob can't be downloaded")

    def is_url_from_local_instance(self):
        """is the url from a local instance or not

        Returns: Boolean

        """
        return self.url_base in SERVER_URI

    def get_url_base(self):
        """get the url base

        Returns: String

        """
        parsed_uri = urlparse(self.url)
        if not parsed_uri.scheme or not parsed_uri.netloc:
            raise exceptions.BlobDownloaderUrlParseError(
                "the url given is not parsable"
            )
        return "{uri.scheme}://{uri.netloc}".format(uri=parsed_uri)



""" Boolean utils test class
"""
from unittest import TestCase

from core_main_app.utils.boolean import to_bool


class TestToBool(TestCase):
    def test_boolean_false_expected_case_returns_false(self):
        self.assertEqual(to_bool("False"), False)

    def test_boolean_true_expected_case_returns_true(self):
        self.assertEqual(to_bool("True"), True)

    def test_boolean_false_unexpected_case_returns_false(self):
        self.assertEqual(to_bool("false"), False)

    def test_boolean_true_unexpected_case_returns_true(self):
        self.assertEqual(to_bool("true"), True)

    def test_boolean_false_returns_false(self):
        self.assertEqual(to_bool(False), False)

    def test_boolean_true_returns_true(self):
        self.assertEqual(to_bool(True), True)

    def test_unexpected_string_raises_value_error(self):
        with self.assertRaises(ValueError):
            to_bool("bool")

    def test_unexpected_type_raises_value_error(self):
        with self.assertRaises(ValueError):
            to_bool(1)



""" Set of functions to define the common rules for access control across collections
"""
import logging

from django.contrib.auth.models import User

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.workspace import api as workspace_api
from core_main_app.permissions import api as permissions_api, rights as rights
from core_main_app.settings import (
    CAN_SET_PUBLIC_DATA_TO_PRIVATE,
    CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT,
)

logger = logging.getLogger(__name__)


def has_perm_publish(user, codename):
    """Does the user have the permission to publish.

    Args:
        user
        codename

    Returns
    """
    publish_perm = permissions_api.get_by_codename(codename)
    if not user.has_perm(
        publish_perm.content_type.app_label + "." + publish_perm.codename
    ):
        raise AccessControlError("The user doesn't have enough rights to publish.")


def has_perm_administration(func, *args, **kwargs):
    """Does the given user have administration rights.

    Args:
        func:
        *args:
        **kwargs:

    Returns:

    """
    try:
        user = next((arg for arg in args if isinstance(arg, User)), None)
        if user and user.is_superuser:
            return func(*args, **kwargs)
    except Exception as e:
        logger.warning("has_perm_administration threw an exception: ".format(str(e)))

    raise AccessControlError("The user doesn't have enough rights.")


def is_superuser(func, *args, **kwargs):
    """Is the user a superuser.

    Args:
        func:
        *args:
        **kwargs:

    Returns:

    """
    try:
        request = kwargs["request"]
        if request and request.user.is_superuser:
            return func(*args, **kwargs)
    except Exception as e:
        logger.warning("has_perm_administration threw an exception: ".format(str(e)))

    raise AccessControlError("The user doesn't have enough rights.")


def check_can_write(document, user):
    """Check that the user can write.

    Args:
        document:
        user:

    Returns:

    """
    # TODO: data will inherit of workspace rights, which means a owner can't edit or delete a data if data in wkp that doesn't give hin write rights
    if hasattr(document, "workspace") and document.workspace is not None:
        if workspace_api.is_workspace_public(document.workspace):
            has_perm_publish(user, rights.publish_data)
        else:
            _check_can_write_in_workspace(document.workspace, user)

    # not the owner and workspace is not set or None
    if document.user_id != str(user.id) and (
        not hasattr(document, "workspace") or document.workspace is None
    ):
        raise AccessControlError("The user doesn't have enough rights.")


def check_can_read_list(document_list, user):
    """Check that the user can read each document of the list.

    Args:
        document_list:
        user:

    Returns:

    """
    if len(document_list) > 0:
        # get list of accessible workspaces
        accessible_workspaces = (
            workspace_api.get_all_workspaces_with_read_access_by_user(user)
        )
        # check access is correct
        for document in document_list:
            # user is document owner
            if document.user_id == str(user.id):
                continue
            # user is not owner or document not in accessible workspace
            if (
                document.workspace is None
                or document.workspace not in accessible_workspaces
            ):
                raise AccessControlError("The user doesn't have enough rights.")


def can_write_document_in_workspace(func, document, workspace, user):
    """Can user write data in workspace.

    Args:
        func:
        document:
        workspace:
        user:

    Returns:

    """
    return can_write_in_workspace(func, document, workspace, user, rights.publish_data)


def can_read_or_write_in_workspace(func, workspace, user):
    """Can user read or write in workspace.

    Args:
        func:
        workspace:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(workspace, user)

    _check_can_read_or_write_in_workspace(workspace, user)
    return func(workspace, user)


def can_write_in_workspace(func, document, workspace, user, codename):
    """Can user write in workspace.

    Args:
        func:
        document:
        workspace:
        user:
        codename:

    Returns:

    """
    if user.is_superuser:
        return func(document, workspace, user)
    if workspace is not None:
        if workspace_api.is_workspace_public(workspace):
            has_perm_publish(user, codename)
        else:
            _check_can_write_in_workspace(workspace, user)

    check_can_write(document, user)

    # if we can not unpublish
    if CAN_SET_PUBLIC_DATA_TO_PRIVATE is False:
        # if document is in public workspace
        if document.workspace is not None and workspace_api.is_workspace_public(
            document.workspace
        ):
            # if target workspace is private
            if (
                workspace is None
                or workspace_api.is_workspace_public(workspace) is False
            ):
                raise AccessControlError("The document can not be unpublished.")

    return func(document, workspace, user)


def can_read(func, user):
    """Can a user read

    Args:
        func:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(user)

    # get list of document
    document_list = func(user)
    # check that the user can access the list of document
    check_can_read_list(document_list, user)
    # return list of document
    return document_list


def can_read_id(func, document_id, user):
    """Can read from object id.

    Args:
        func:
        document_id:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(document_id, user)

    document = func(document_id, user)
    _check_can_read(document, user)
    return document


def can_write(func, document, user):
    """Can user write

    Args:
        func:
        document:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(document, user)

    check_can_write(document, user)
    return func(document, user)


def can_request_write(func, document, request):
    """Can user request write

    Args:
        func:
        document:
        request:

    Returns:

    """
    if request.user.is_superuser:
        return func(document, request)

    check_can_write(document, request.user)
    return func(document, request)


def can_anonymous_access_public_data(func, *args, **kwargs):
    """Can anonymous access a public data

    Args:
        func:
        *args:
        **kwargs:

    Returns:

    """
    user = next((arg for arg in args if isinstance(arg, User)), None)

    _check_anonymous_access(user)
    return func(*args, **kwargs)


def _check_can_write_in_workspace(workspace, user):
    """Check that user can write in the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    accessible_workspaces = workspace_api.get_all_workspaces_with_write_access_by_user(
        user
    )
    if workspace not in accessible_workspaces:
        raise AccessControlError(
            "The user does not have the permission to write into this workspace."
        )


def _check_can_read_or_write_in_workspace(workspace, user):
    """Check that user can read or write in the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    accessible_write_workspaces = (
        workspace_api.get_all_workspaces_with_write_access_by_user(user)
    )
    accessible_read_workspaces = (
        workspace_api.get_all_workspaces_with_read_access_by_user(user)
    )
    if workspace not in list(accessible_write_workspaces) + list(
        accessible_read_workspaces
    ):
        raise AccessControlError(
            "The user does not have the permission to read or write into this workspace."
        )


def _check_anonymous_access(user):
    """Check anonymous access

    Args:
        user:

    Returns:
    """
    if (user is None or user.is_anonymous) and not CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
        raise AccessControlError(
            "The user doesn't have enough rights to access this document."
        )


def _check_can_read(document, user):
    """Check that the user can read.

    Args:
        document:
        user:

    Returns:

    """
    _check_anonymous_access(user)

    # workspace case
    if document.user_id != str(user.id):
        # workspace is set
        if hasattr(document, "workspace") and document.workspace is not None:
            # get list of accessible workspaces
            accessible_workspaces = (
                workspace_api.get_all_workspaces_with_read_access_by_user(user)
            )
            # check that accessed document belongs to an accessible workspace
            if document.workspace not in accessible_workspaces:
                raise AccessControlError(
                    "The user doesn't have enough rights to access this."
                )
        # workspace is not set
        else:
            raise AccessControlError(
                "The user doesn't have enough rights to access this."
            )


def can_change_owner(func, document, new_user, user):
    """Can user change document's owner.

    Args:
        func:
        document:
        new_user:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(document, new_user, user)

    if document.user_id != str(user.id):
        raise AccessControlError(
            "The user doesn't have enough rights to access this document."
        )

    return func(document, new_user, user)



"""Forms for admin views
"""
from django import forms
from django_mongoengine.forms import DocumentForm

from core_main_app.commons.validators import ExtensionValidator
from core_main_app.components.template import api as template_api
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.xsl_transformation import api as xsl_transformation_api
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.views.admin.commons.upload.forms import UploadForm


class EditTemplateForm(DocumentForm):
    title = forms.CharField(
        label="Name",
        widget=forms.TextInput(
            attrs={"class": "form-control", "placeholder": "Type the new name"}
        ),
    )

    class Meta(object):
        document = TemplateVersionManager
        fields = ["title"]


class EditXSLTForm(DocumentForm):
    name = forms.CharField(
        label="Name",
        widget=forms.TextInput(
            attrs={"class": "form-control", "placeholder": "Type the new name"}
        ),
    )

    class Meta(object):
        document = XslTransformation
        fields = ["name"]


class UploadTemplateForm(UploadForm):
    """
    Form to upload a new Template
    """

    def __init__(self, *args, **kwargs):
        super(UploadTemplateForm, self).__init__(*args, **kwargs)
        self.fields["name"].label = "Enter Template name"
        self.fields["upload_file"].widget = forms.FileInput(attrs={"accept": ".xsd"})


class UploadVersionForm(forms.Form):
    """
    Form to upload a new version
    """

    xsd_file = forms.FileField(
        label="Select a file",
        required=True,
        widget=forms.FileInput(attrs={"accept": ".xsd"}),
    )


class EditProfileForm(forms.Form):
    """
    Form to edit the profile information
    """

    firstname = forms.CharField(label="First Name", max_length=100, required=True)
    lastname = forms.CharField(label="Last Name", max_length=100, required=True)
    username = forms.CharField(
        label="Username", max_length=100, required=True, widget=forms.HiddenInput()
    )
    email = forms.EmailField(label="Email Address", max_length=100, required=True)


class UploadXSLTForm(UploadForm):
    """
    Form to upload a new XSLT
    """

    def __init__(self, *args, **kwargs):
        super(UploadXSLTForm, self).__init__(*args, **kwargs)
        self.fields["name"].label = "Enter XSLT name"
        self.fields["upload_file"].validators = [ExtensionValidator(".xsl")]
        self.fields["upload_file"].widget = forms.FileInput(
            attrs={"accept": ".xslt, .xsl"}
        )


class TemplateXsltRenderingForm(forms.Form):
    """
    Form to associate list and detail XSLTs to template
    """

    id = forms.CharField(widget=forms.HiddenInput(), required=False)
    template = forms.CharField(widget=forms.HiddenInput(), required=False)

    list_xslt = forms.ModelChoiceField(
        label="Set XSLT to render a list of data",
        empty_label="(No XSLT)",
        required=False,
        widget=forms.Select(attrs={"class": "form-control"}),
        queryset=xsl_transformation_api.get_all(),
    )

    list_detail_xslt = forms.MultipleChoiceField(
        label="Set XSLT to render a single data",
        required=False,
        widget=forms.widgets.CheckboxSelectMultiple,
    )

    default_detail_xslt = forms.ModelChoiceField(
        label="Default rendering",
        empty_label="(No XSLT)",
        required=False,
        widget=forms.Select(attrs={"class": "form-control"}),
        queryset=xsl_transformation_api.get_all(),
    )

    def __init__(self, *args, **kwargs):
        super(TemplateXsltRenderingForm, self).__init__(*args, **kwargs)
        self.fields["list_detail_xslt"].choices = _get_xsl_transformation()


class TextAreaForm(forms.Form):
    """TextArea Form"""

    content = forms.CharField(
        label="", widget=forms.Textarea(attrs={"class": "form-control"}), required=False
    )


def _get_xsl_transformation():
    """Get XSLT.

    Returns:
        List of XSLT.

    """
    xsl_transformation = []
    list_ = xsl_transformation_api.get_all()
    for elt in list_:
        xsl_transformation.append((elt.id, elt.name))
    return xsl_transformation



""" Blob tags
"""
import re

from django import template

from core_main_app.utils.urls import get_blob_download_regex

register = template.Library()


@register.simple_tag(name="render_blob_links_in_span")
def render_blob_links_in_span(*args, **kwargs):
    """Find all blobs link then frame then with <span>

    Args:
        *args:
        **kwargs:

    Returns:

    """
    xml_string = kwargs["xml_string"]
    # get all blobs link
    url_blobs = re.findall(get_blob_download_regex(), xml_string)
    # we attend to frame then with a specific class selector
    blob_html_pattern = "<span class='blob-link' data-blob-url=\"{0}\">{0}</span>"
    # for all urls found, we apply the pattern
    for url in url_blobs:
        xml_string = xml_string.replace(url, blob_html_pattern.format(url))
    return xml_string



""" Blob utils test class
"""
import unittest
from unittest import TestCase

import mock
from django.conf import settings
from mock import patch

from core_main_app.commons.exceptions import BlobDownloaderUrlParseError
from core_main_app.utils.blob_downloader import BlobDownloader
from tests.test_settings import SERVER_URI


class TestBlobDownloaderIsUrlFormLocalInstance(TestCase):
    def test_is_url_from_local_instance_returns_false_if_url_is_not_from_local_instance(
        self,
    ):
        # Arrange / Act
        return_value = BlobDownloader("http://google.com").is_url_from_local_instance()
        # Assert
        self.assertEqual(return_value, False)

    def test_is_url_from_local_instance_returns_true_if_url_is_from_local_instance(
        self,
    ):
        # Arrange / Act
        return_value = BlobDownloader(
            "http://example.com/987653456789"
        ).is_url_from_local_instance()
        # Assert
        self.assertEqual(return_value, True)


class TestBlobDownloaderGetUrlBase(TestCase):
    def test_get_url_base_returns_url_base(self):
        # Arrange / Act
        return_value = BlobDownloader("http://example.com/987653456789").get_url_base()
        # Assert
        self.assertEqual(return_value, SERVER_URI)

    def test_get_url_base_raise_exception_if_url_is_not_parsable(self):
        # Assert
        with self.assertRaises(BlobDownloaderUrlParseError):
            # Arrange / Act
            BlobDownloader("random_string").get_url_base()


class TestBlobDownloaderGetBlobResponse(TestCase):
    def test_get_blob_response_raise_exception_if_url_is_unidentified(self):
        # Assert
        with self.assertRaises(BlobDownloaderUrlParseError):
            # Arrange / Act
            BlobDownloader("random_string").get_blob_response()

    @patch("core_main_app.utils.requests_utils.requests_utils.send_get_request")
    def test_get_blob_response_return_blob_from_local_if_url_is_local(
        self, mock_send_get_request
    ):
        # Arrange / Act
        mock_send_get_request.return_value = "local"
        return_value = BlobDownloader(
            "http://example.com/987653456789"
        ).get_blob_response()
        # Assert
        self.assertEqual(return_value, "local")

    @unittest.skip("How to mock the import done in the API?")
    @patch(
        "core_federated_search_app.components.instance.api.get_blob_response_from_url"
    )
    def test_get_blob_response_return_blob_from_federated_remote_if_url_is_identified_as_remote_federated(
        self, mock_send_get_request
    ):
        # Arrange / Act
        mock_send_get_request.return_value = "remote"
        with mock.patch.object(
            settings, "INSTALLED_APPS", ["core_federated_search_app"]
        ):
            return_value = BlobDownloader(
                "http://my.remote.com/987653456789"
            ).get_blob_response()
        # Assert
        self.assertEqual(return_value, "remote")



"""Access control decorator
"""


def access_control(check_func):
    """Access control decorator.

    Args:
        check_func: function that checks access

    Returns:

    """

    def _access_control(func):
        """Decorator.

        Args:
            func: function on which the decorator is attached.

        Returns:

        """

        def wrapper(*args, **kwargs):
            """Function wrapper.

            Args:
                *args:
                **kwargs:

            Returns:

            """
            return check_func(func, *args, **kwargs)

        return wrapper

    return _access_control



""" Form needed for the user part of everything
"""
from django import forms

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.user import api as user_api
from core_main_app.components.workspace import api as workspace_api
from core_main_app.utils.labels import get_data_label


class LoginForm(forms.Form):
    """Custom login form for the user"""

    username = forms.CharField(label="Username", required=True)
    password = forms.CharField(
        label="Password", required=True, widget=forms.PasswordInput
    )

    next_page = forms.CharField(widget=forms.HiddenInput)


class GroupRightForm(forms.Form):
    """
    Form to select group to add rights.
    """

    groups = forms.MultipleChoiceField(
        label="", required=True, widget=forms.SelectMultiple()
    )
    GROUPS_OPTIONS = []

    def __init__(self, groups_with_no_access):
        self.GROUPS_OPTIONS = []

        # We sort by name, case sensitive
        sort_groups = sorted(groups_with_no_access, key=lambda s: s.name.lower())

        # We add them
        for group in sort_groups:
            self.GROUPS_OPTIONS.append((group.id, group.name))

        super(GroupRightForm, self).__init__()
        self.fields["groups"].choices = []
        self.fields["groups"].choices = self.GROUPS_OPTIONS


class UserRightForm(forms.Form):
    """
    Form to select user to add rights.
    """

    users = forms.MultipleChoiceField(
        label="", required=True, widget=forms.SelectMultiple()
    )
    USERS_OPTIONS = []

    def __init__(self, users_with_no_access):
        self.USERS_OPTIONS = []

        # We sort by username, case sensitive
        sort_users = sorted(users_with_no_access, key=lambda s: s.username.lower())

        # We add them
        for user in sort_users:
            self.USERS_OPTIONS.append((user.id, user.username))

        super(UserRightForm, self).__init__()
        self.fields["users"].choices = []
        self.fields["users"].choices = self.USERS_OPTIONS


class WorkspaceForm(forms.Form):
    """
    Form to create the workspace.
    """

    workspace_name = forms.CharField(max_length=100)


class ChangeWorkspaceForm(forms.Form):
    """
    Form to select a workspace.
    """

    workspaces = forms.ChoiceField(
        label="", required=True, widget=forms.Select(attrs={"class": "form-control"})
    )
    WORKSPACES_OPTIONS = []

    def __init__(
        self,
        user,
        list_current_workspace=[],
        is_administration=False,
        show_global_workspace=False,
    ):
        self.WORKSPACES_OPTIONS = []
        self.WORKSPACES_OPTIONS.append(("", "-----------"))

        # We retrieve all workspaces with write access, or all workspaces if administration
        if is_administration:
            all_workspaces = workspace_api.get_all()
        else:
            all_workspaces = list(
                workspace_api.get_all_workspaces_with_write_access_by_user(user)
            )
            if show_global_workspace:
                workspace_global = workspace_api.get_global_workspace()
                if workspace_global not in all_workspaces:
                    all_workspaces.append(workspace_global)

        if len(all_workspaces) == 0:
            raise DoesNotExist(
                "You don't have access to any workspaces with sufficient rights to assign a "
                + get_data_label()
                + "."
            )

        # We sort by title, case insensitive
        sort_workspaces = sorted(all_workspaces, key=lambda s: s.title.lower())

        # We add them
        for workspace in sort_workspaces:
            is_workspace_global = workspace_api.is_workspace_global(workspace)
            if (
                list_current_workspace == []
                or (
                    len(list_current_workspace) > 0
                    and workspace not in list_current_workspace
                )
            ) and (
                (show_global_workspace and is_workspace_global)
                or not is_workspace_global
            ):

                self.WORKSPACES_OPTIONS.append(
                    (
                        workspace.id,
                        workspace.title
                        + " ("
                        + (
                            "GLOBAL"
                            if is_workspace_global
                            else user_api.get_user_by_id(workspace.owner).username
                        )
                        + ")",
                    )
                )

        super(ChangeWorkspaceForm, self).__init__()
        self.fields["workspaces"].choices = []
        self.fields["workspaces"].choices = self.WORKSPACES_OPTIONS



""" Url router for the main application
"""
from django.conf.urls import include
from django.urls import re_path
from drf_yasg import openapi
from drf_yasg.views import get_schema_view

from core_main_app.components.blob import api as blob_api
from core_main_app.components.data import api as data_api
from core_main_app.utils.rendering import render
from core_main_app.views.common import ajax as common_ajax, views as common_views
from core_main_app.views.user import views as user_views, ajax as user_ajax

schema_view = get_schema_view(
    openapi.Info(
        title="REST API",
        default_version="v1",
    ),
)

urlpatterns = [
    re_path(r"^$", user_views.homepage, name="core_main_app_homepage"),
    re_path(r"^login", user_views.custom_login, name="core_main_app_login"),
    re_path(r"^rest/", include("tests.rest_urls")),
    re_path(
        r"^data", common_views.ViewData.as_view(), name="core_main_app_data_detail"
    ),
    re_path(
        r"^template/versions/(?P<version_manager_id>\w+)",
        user_views.manage_template_versions,
        name="core_main_app_manage_template_versions",
    ),
    re_path(
        r"^template/(?P<pk>[\w-]+)/edit/$",
        common_ajax.EditTemplateVersionManagerView.as_view(),
        name="core_main_app_edit_template",
    ),
    re_path(
        r"^template/xslt/(?P<template_id>\w+)",
        common_views.TemplateXSLRenderingView.as_view(
            rendering=render,
            template_name="core_main_app/common/templates_xslt/main.html",
            save_redirect="core_main_app_manage_template_versions",
        ),
        name="core_main_app_template_xslt",
    ),
    re_path(
        r"^edit-rights/(?P<workspace_id>\w+)$",
        common_views.EditWorkspaceRights.as_view(),
        name="core_main_edit_rights_workspace",
    ),
    re_path(
        r"^change-workspace",
        user_ajax.LoadFormChangeWorkspace.as_view(),
        name="core_main_change_workspace",
    ),
    re_path(
        r"^assign-blob-workspace",
        user_ajax.AssignView.as_view(api=blob_api),
        name="core_main_assign_blob_workspace",
    ),
    re_path(
        r"^assign-data-workspace",
        user_ajax.AssignView.as_view(api=data_api),
        name="core_main_assign_data_workspace",
    ),
    re_path(
        r"^add-user-form",
        user_ajax.load_add_user_form,
        name="core_main_edit_rights_users_form",
    ),
    re_path(
        r"^add-user-right-to-workspace",
        user_ajax.add_user_right_to_workspace,
        name="core_main_add_user_right_to_workspace",
    ),
    re_path(r"^switch-right", user_ajax.switch_right, name="core_main_switch_right"),
    re_path(
        r"^remove-rights",
        user_ajax.remove_user_or_group_rights,
        name="core_main_remove_rights",
    ),
    re_path(
        r"^add-group-form",
        user_ajax.load_add_group_form,
        name="core_main_edit_rights_groups_form",
    ),
    re_path(
        r"^add-group-right-to-workspace",
        user_ajax.add_group_right_to_workspace,
        name="core_main_add_group_right_to_workspace",
    ),
    re_path(r"^tz_detect/", include("tz_detect.urls")),
    re_path(
        r"^change-data-display",
        user_ajax.change_data_display,
        name="core_main_add_change_data_display",
    ),
]



"""
Version Manager API
"""
from core_main_app.access_control.decorators import access_control
from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import ApiError
from core_main_app.components.version_manager.access_control import (
    can_read,
    can_write,
    can_read_global,
    can_read_list,
    can_read_version_manager,
)
from core_main_app.components.version_manager.models import VersionManager


@access_control(can_read)
def get(version_manager_id, request):
    """Get a version manager by its id.

    Args:
        version_manager_id:
        request:

    Returns:

    """
    return VersionManager.get_by_id(version_manager_id)


@access_control(can_read_list)
def get_by_id_list(list_id, request):
    """Get a version managers with the given id list.

    Args:
        list_id:
        request

    Returns:

    """
    return VersionManager.get_by_id_list(list_id)


@access_control(can_read)
def get_from_version(version, request):
    """Return a version manager from a version.

    Args:
        version:
        request:

    Returns:

    """
    version_managers = VersionManager.get_all()
    for version_manager in version_managers:
        if str(version.id) in version_manager.versions:
            return version_manager
    raise exceptions.ApiError("No version manager could be found for this version.")


# NOTE: access control by upsert
def disable(version_manager, request):
    """Disable an object, then saves it.

    Args:
        version_manager:
        request:

    Returns:

    """
    version_manager.disable()
    return upsert(version_manager, request=request)


# NOTE: access control by upsert
def restore(version_manager, request):
    """Restore an object, then saves it.

    Args:
        version_manager:
        request:

    Returns:

    """
    version_manager.restore()
    return upsert(version_manager, request=request)


# NOTE: access control by upsert
def restore_version(version, request):
    """Disable a version of the object, then saves it.

    Args:
        version:
        request:

    Returns:

    """
    version_manager = get_from_version(version, request=request)
    try:
        version_manager.restore_version(version)
    except ValueError:
        raise ApiError("Unable to restore this version: status is not disabled.")
    return upsert(version_manager, request=request)


# NOTE: access control by upsert
def disable_version(version, request, new_current=None):
    """Disable a version of the object, then saves it.

    Args:
        version:
        new_current:
        request:

    Returns:

    """
    version_manager = get_from_version(version, request=request)

    # try to delete the current version
    if version_manager.current == str(version.id):
        # no version to be current provided
        if new_current is None:
            raise exceptions.ApiError("Unable to disable the current version.")

        # id doesn't match a version
        if new_current.id not in version_manager.versions:
            raise exceptions.ApiError(
                "The id provided to be the next current version, could not be found."
            )

        # set the new current version
        version_manager.set_current_version(new_current)

    # disable the version
    version_manager.disable_version(version)
    return upsert(version_manager, request=request)


# NOTE: access control by upsert
def set_current(version, request):
    """Set the current version of the object, then saves it.

    Args:
        version:
        request:

    Returns:

    """
    version_manager = get_from_version(version, request=request)

    # a disabled version cannot be current
    if str(version.id) in version_manager.get_disabled_versions():
        raise exceptions.ApiError(
            "Unable to set the current version because it is disabled."
        )

    version_manager.set_current_version(version)
    return upsert(version_manager, request=request)


@access_control(can_write)
def upsert(version_manager, request):
    """Save or update version manager.

    Args:
        version_manager:
        request:

    Returns:

    """
    return version_manager.save_version_manager()


# NOTE: access control by upsert
def insert_version(version_manager, version, request):
    """Insert a version in the version manager, then saves it.

    Args:
        version_manager:
        version:
        request:

    Returns:

    """
    version_manager.insert(version)

    # first version inserted, set it as current
    if len(version_manager.versions) == 1:
        version_manager.set_current_version(version)

    return upsert(version_manager, request=request)


@access_control(can_read_global)
def get_active_global_version_manager_by_title(version_manager_title, request):
    """Return active Version Manager by its title with user set to None.

    Args:
        version_manager_title: Version Manager title
        request:

    Returns:
        Version Manager instance

    """
    return VersionManager.get_active_global_version_manager_by_title(
        version_manager_title
    )


@access_control(can_read_version_manager)
def get_version_number(version_manager, version, request):
    """Return version number from version id.

    Args:
        version_manager:
        version:
        request:

    Returns:

    """
    return version_manager.versions.index(str(version)) + 1


@access_control(can_read_version_manager)
def get_version_by_number(version_manager, version_number, request):
    """Return the version by its version number.

    Args:
        version_manager:
        version_number: Number of the version
        request:

    Returns:

    """
    return version_manager.versions[version_number - 1]



""" Method's decorators
"""

from functools import wraps
from urllib.parse import urlparse

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.models import Group, User
from django.core.exceptions import PermissionDenied
from django.db.models import Q
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.shortcuts import resolve_url
from rest_framework import status
from rest_framework.response import Response

import core_main_app.permissions.rights as rights


@receiver(post_save, sender=User)
def add_default_group(sender, **kwargs):
    """
    # Function Name: add_default_group(request)
    # Inputs:        sender, kwargs -
    # Outputs:       -
    # Exceptions:    None
    # Description:   Assigns the user in the default group after its creation
    #                We catch the post_save signal for the User model

    :param sender:
    :param kwargs:
    :return:
    """
    # We retrieve the user
    user = kwargs["instance"]
    # If it's a creation
    if kwargs["created"]:
        group = Group.objects.get(name=rights.default_group)
        user.groups.add(group)
        user.save()


def login_or_anonymous_perm_required(
    anonymous_permission,
    function=None,
    redirect_field_name=REDIRECT_FIELD_NAME,
    login_url=None,
):
    """
    # Function Name: login_or_anonymous_perm_required(request)
    # Inputs:        anonymous_permission, function=None, redirect_field_name=REDIRECT_FIELD_NAME, login_url=None -
    # Outputs:       decorator
    # Exceptions:    None
    # Description:   Custom decorator for checking user authentication or anonymous user permission.
    #                Manages the authorisation to execute a function decorated by this decorator.
    #                Conditions: user connected or anonymous_group assigned with the anonymous_permission in parameter.

    :param anonymous_permission:
    :param function:
    :param redirect_field_name:
    :param login_url:
    :return:
    """

    def _check_group(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            """

            Args:
                request:
                *args:
                **kwargs:

            Returns:

            """
            request.has_anonymous_access = False
            if request.user.is_anonymous:
                access = Group.objects.filter(
                    Q(name=rights.anonymous_group)
                    & Q(permissions__codename=anonymous_permission)
                )
            else:
                access = request.user.is_authenticated

            if access:
                return view_func(request, *args, **kwargs)
            else:
                path = request.build_absolute_uri()
                resolved_login_url = resolve_url(login_url or settings.LOGIN_URL)
                # If the login url is the same scheme and net location then just
                # use the path as the "next" url.
                login_scheme, login_netloc = urlparse(resolved_login_url)[:2]
                current_scheme, current_netloc = urlparse(path)[:2]
                if (not login_scheme or login_scheme == current_scheme) and (
                    not login_netloc or login_netloc == current_netloc
                ):
                    path = request.get_full_path()
                from django.contrib.auth.views import redirect_to_login

                return redirect_to_login(path, resolved_login_url, redirect_field_name)

        return wrapper

    return _check_group


def permission_required(
    content_type,
    permission,
    login_url=None,
    raise_exception=False,
    redirect_field_name=REDIRECT_FIELD_NAME,
):
    """
    # Function Name: permission_required(request)
    # Inputs:        content_type, permission, login_url=None, raise_exception=False,
    #                redirect_field_name=REDIRECT_FIELD_NAME -
    # Outputs:       decorator
    # Exceptions:    None
    # Description:   Check if the user has the required permission given in parameter
    #                If the user is anonymous, check if the anonymous_group has the required permission

    :param content_type:
    :param permission:
    :param login_url:
    :param raise_exception:
    :param redirect_field_name:
    :return:
    """

    def _check_group(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            """

            Args:
                request:
                *args:
                **kwargs:

            Returns:

            """
            if request.user.is_anonymous:
                # Check in the anonymous_group
                access = Group.objects.filter(
                    Q(name=rights.anonymous_group) & Q(permissions__codename=permission)
                )
            else:
                # Check the permission for the current user
                prefixed_permission = "{!s}.{!s}".format(content_type, permission)
                access = request.user.has_perm(prefixed_permission)

            if access:
                return view_func(request, *args, **kwargs)
            else:
                # In case the 403 handler should be called raise the exception
                if raise_exception:
                    raise PermissionDenied
                else:
                    path = request.build_absolute_uri()
                    resolved_login_url = resolve_url(login_url or settings.LOGIN_URL)
                    # If the login url is the same scheme and net location then just
                    # use the path as the "next" url.
                    login_scheme, login_netloc = urlparse(resolved_login_url)[:2]
                    current_scheme, current_netloc = urlparse(path)[:2]
                    if (not login_scheme or login_scheme == current_scheme) and (
                        not login_netloc or login_netloc == current_netloc
                    ):
                        path = request.get_full_path()
                    from django.contrib.auth.views import redirect_to_login

                    return redirect_to_login(
                        path, resolved_login_url, redirect_field_name
                    )

        return wrapper

    return _check_group


def api_staff_member_required():
    """
    # Function Name: api_staff_member_required(request)
    # Inputs:        -
    # Outputs:       decorator
    # Exceptions:    None
    # Description:   Check if the user is an admin user. Used by the API

    :return:
    """

    def _check_group(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            """

            Args:
                request:
                *args:
                **kwargs:

            Returns:

            """
            if request.user.is_staff:
                return view_func(request, *args, **kwargs)
            else:
                content = {"message": "Only administrators can use this feature."}
                return Response(content, status=status.HTTP_403_FORBIDDEN)

        return wrapper

    return _check_group


def api_permission_required(content_type, permission, raise_exception=False):
    """
    # Function Name: api_permission_required(request)
    # Inputs:        -
    # Outputs:       decorator
    # Exceptions:    None
    # Description:   Check if the user has the required permission given in parameter.
    #                If the user is anonymous, check if the anonymous_group has the required permission
    #                Used by the API

    :param content_type:
    :param permission:
    :param raise_exception:
    :return:
    """

    def _check_group(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            """
            Args:
                request:
                *args:
                **kwargs:

            Returns:

            """
            if request.user.is_anonymous:
                access_api = Group.objects.filter(
                    Q(name=rights.anonymous_group)
                    & Q(permissions__codename=rights.api_access)
                )
                access = Group.objects.filter(
                    Q(name=rights.anonymous_group) & Q(permissions__codename=permission)
                )
            else:
                prefixed_api_permission = "{!s}.{!s}".format(
                    rights.api_content_type, rights.api_access
                )
                access_api = request.user.has_perm(prefixed_api_permission)
                prefixed_permission = "{!s}.{!s}".format(content_type, permission)
                access = request.user.has_perm(prefixed_permission)

            if access_api and access:
                return view_func(request, *args, **kwargs)
            else:
                # In case the 403 handler should be called raise the exception
                if raise_exception:
                    raise PermissionDenied
                else:
                    content = {
                        "message": "You don't have enough rights to use this feature."
                    }
                    return Response(content, status=status.HTTP_403_FORBIDDEN)

        return wrapper

    return _check_group
